searchNodes=[{"doc":"Endo is a library which allows you to reflect on the tables, columns, indexes, and other data stored within a given Ecto Repo. Basic Usage To get started with Endo, all you need is a supported Ecto Repo from your application, as well as the Endo dependency. Currently, Endo only supports the Ecto.Adapters.Postgres adapter, but Endo was built to be adapter agnostic, so additional adapters may be implemented in the future. Given an Ecto Repo whose adapter is Ecto.Adapters.Postgres , you can execute Endo with the following snippet: Endo . list_tables ( MyApp.Repo ) This default invokation will return a list of all public tables in your repo's database, as well as metadata pertaining to said tables. At the time of writing, table metadata includes: The name of the given table. A list of columns defined in the given table. A list of indexes defined in the given table. A list of associations, if any, sourced from the given table. The repo adapter, for reference. In the future, we plan on implementing a list of constraints, as well as surfacing additional stats and metrics which we can lift from the underlying adapters. Note By default, Endo will return all tables in the public schema for a Postgres repo. This will include a default schema_migrations table which is seeded by Ecto itself. Additionally, if you're using dependencies such as Oban , any tables created for these dependencies may also be returned. Complex Lookups By providing the optional second paramter to Endo.list_tables/2 , you can instruct Endo to fetch tables matching an arbitrary set of criterion. Options are ultimately passed down into the Endo adapter matching the given repo module, but in general the supported high-level options are: with_column , instructs Endo to only return tables which define a column with the given name without_column , instructs Endo to only return tables which do not define a column with the given name with_foreign_key_constraint , instructs Endo to only return tables which have a foreign key constraint to the given table name without_foreign_key_constraint , instructs Endo to only return tables which do not have a foreign key constraint to the given table name with_index , instructs Endo to only return tables with indexes on the given column or columns. If a list of columns if given, then we look for a compound index that exactly matches the given ones components and order. without_index , instructs Endo to only return tables without indexes on the given column or columns. Likewise, compound indexes must exactly match the given ones components and order. Additionally, adapters are free to implement custom filters. The Endo.Adapters.Postgres adapter forwards any filters not matching the above list directly to direct SQL queries against the base information schema tables the adapter sources from. This means that is it possible to perform direct queries such as Endo.list_tables(MyApp.Repo, table_name: &quot;payments&quot;) . See Endo.Adapters.Postgres and Endo.Adapters.Postgres.Table for a list of supported fields -- though as these are internal to Endo, do not count on these being stable across version upgrades. Endo also supports multiple filters being given in a single invokation. Multiple filters will apply in addition to one another, in the order which they are given. The same filter can be provided multiple times also. See the following example for details: Endo . list_tables ( MyApp.Repo , with_column : &quot;inserted_at&quot; ) # Returns list of tables defining an `inserted_at` column. Endo . list_tables ( MyApp.Repo , with_column : &quot;inserted_at&quot; , without_index : &quot;inserted_at&quot; ) # Returns list of tables defining an `inserted_at` column that does not index said column. Endo . list_tables ( MyApp.Repo , with_column : &quot;inserted_at&quot; , without_column : &quot;updated_at&quot; ) # Returns list of tables defining an `inserted_at` column while also not defining an `updated_at` column. Endo . list_tables ( MyApp.Repo , with_foreign_key_constraint : &quot;cars&quot; , without_index : &quot;car_id&quot; ) # Returns list of tables defining a relation to `cars`, without an index on said column Endo . list_tables ( MyApp.Repo , with_index : [ &quot;org_id&quot; , &quot;location_id&quot; , &quot;patient_id&quot; ] ) # Returns list of tables defining a *compound index* on `(org_id, location_id, patient_id)` # Tables containing the same index in a different order, or a partial match will not be returned.","ref":"Endo.html","title":"Endo","type":"module"},{"doc":"Given an Ecto Repo and a table name, tries to return an Endo Table or nil if said table does not exist. Internally delegates to list_tables/2 with the table_name option set. See list_tables/2 for more information.","ref":"Endo.html#get_table/3","title":"Endo.get_table/3","type":"function"},{"doc":"Given an Ecto Repo, returns a list of all tables, columns, associations, and indexes. Takes an optional keyword list of filters which filter said tables down to given constraints.","ref":"Endo.html#list_tables/2","title":"Endo.list_tables/2","type":"function"},{"doc":"Given a list of Endo Tables or a single Endo Table, tries to load the application-specific Ecto Schemas See Endo.Schema.load/1 for more information.","ref":"Endo.html#load_schemas/1","title":"Endo.load_schemas/1","type":"function"},{"doc":"Module defining the Endo.Adapter behaviour. Valid adapters will allow Endo to reflect upon a module implementing said adater","ref":"Endo.Adapter.html","title":"Endo.Adapter","type":"behaviour"},{"doc":"","ref":"Endo.Adapter.html#c:list_tables/2","title":"Endo.Adapter.list_tables/2","type":"callback"},{"doc":"","ref":"Endo.Adapter.html#c:to_endo/1","title":"Endo.Adapter.to_endo/1","type":"callback"},{"doc":"Adapter module implementing the ability for Endo to reflect upon any Postgres-based Ecto Repo. See Endo documentation for list of features. In future, parts of Endo 's top level documentation may be moved here, but as this is the only supported adapter at the time of writing, this isn't the case.","ref":"Endo.Adapters.Postgres.html","title":"Endo.Adapters.Postgres","type":"module"},{"doc":"Callback implementation for Endo.Adapter.list_tables/2 .","ref":"Endo.Adapters.Postgres.html#list_tables/2","title":"Endo.Adapters.Postgres.list_tables/2","type":"function"},{"doc":"","ref":"Endo.Adapters.Postgres.html#to_endo/2","title":"Endo.Adapters.Postgres.to_endo/2","type":"function"},{"doc":"Association metadata for a given table's associations","ref":"Endo.Association.html","title":"Endo.Association","type":"module"},{"doc":"","ref":"Endo.Association.html#t:t/0","title":"Endo.Association.t/0","type":"type"},{"doc":"Column metadata for a given table's columns","ref":"Endo.Column.html","title":"Endo.Column","type":"module"},{"doc":"","ref":"Endo.Column.html#t:t/0","title":"Endo.Column.t/0","type":"type"},{"doc":"Index metadata for a given table's indexes","ref":"Endo.Index.html","title":"Endo.Index","type":"module"},{"doc":"","ref":"Endo.Index.html#t:t/0","title":"Endo.Index.t/0","type":"type"},{"doc":"Utility module for discovering Ecto Schema implementations for a given Endo Table","ref":"Endo.Schema.html","title":"Endo.Schema","type":"module"},{"doc":"Given a list of Endo Tables, tries to load their schemas in their corresponding OTP App. All Endo Table structs contain metadata about which OTP app a given Repo belongs to. This information is used to load all Elixir modules that use Ecto.Schema . With this list, we match up any schemas to tables that exist; though it is important to note that not all Endo Tables will necessarily have a corresponding Ecto Schema module defined for it. In this case, the schemas key of an Endo Table will be an empty list. It is also possible for multiple Ecto Schemas to exist for a single underlying database tables, thus, any discovered results will be accumulated and returned as a list of modules per Endo Table.","ref":"Endo.Schema.html#load/1","title":"Endo.Schema.load/1","type":"function"},{"doc":"Table metadata returned by Endo","ref":"Endo.Table.html","title":"Endo.Table","type":"module"},{"doc":"","ref":"Endo.Table.html#t:t/0","title":"Endo.Table.t/0","type":"type"}]